import controlP5.*;
import oscP5.*;
import netP5.*;
import SpringGUI.*;

import java.io.IOException;

import javax.jmdns.*;
import javax.jmdns.ServiceEvent;
import javax.jmdns.ServiceListener;

static int offsetY=130;
ControlP5 controlP5;
static OscP5 oscP5;
static NetAddress myRemoteLocation;

static SpringGUI gui;


static String serviceType="_osc._udp.local.";
static JmDNS jmdns;


Slider markSlider;
Slider posSlider;
Slider volSlider;

// textfield advanced example by andreas schlegel, 11.02.2009
// demonstrates how to use keepFocus, setText, getText, getTextList,
// clear, setAutoClear, isAutoClear and submit.
String textValue = "";
Textfield myTextfield;
Textfield markpos;
Textfield time;
Textfield pos;


static class SampleListener implements ServiceListener {
    OscP5 oscObj;
    SampleListener(OscP5 osc){
      oscObj=osc;
    }
  
 public void serviceAdded(ServiceEvent event) {
   println("Service added   : " + event.getName()+"."+event.getType());
 
      if( !gui.hasItem("deviceList", event.getName()) ){
        gui.addItem("deviceList", event.getName() );
      }
    }
        
        
   public void serviceRemoved(ServiceEvent event) {
          
        println("Service removed : " + event.getName()+"."+event.getType());
         if( gui.hasItem("deviceList", event.getName()) ){
           gui.removeItem("deviceList", event.getName());
         }
   }
        
    public void serviceResolved(ServiceEvent event) {
            println("Service resolved: " + event.getInfo());
          
           ServiceInfo info=event.getInfo();
           if ( gui.isElement("device") ) gui.remove("device");
           if ( gui.isElement("net") ) gui.remove("net");
           gui.addTextField("device", event.getName(), 100, offsetY-10, 380, 20);
           gui.addTextField("net","address:"+info.getHostAddress()+"  port:"+info.getPort(), 100, offsetY+20, 280, 20);
           myRemoteLocation = new NetAddress(info.getHostAddress(),info.getPort());
          
          
          for(int i=0;i<3;i++){
          OscMessage myMessage = new OscMessage("/cbe/ping");
          myMessage.add(0);
          oscP5.send(myMessage, myRemoteLocation); 
          }
           
             
     }
}

void portSet(int port){
  oscP5 = new OscP5(this,port);
}

void setup() {
  size(700,800);
  frameRate(25);
  
 
  // load a font to use for text-output...
 // PFont font = loadFont("TrebuchetMS-11.vlw"); 
 // textFont(font);
  
    oscP5 = new OscP5(this,10000);
    myRemoteLocation = new NetAddress("192.168.0.2",10000);
  
  controlP5 = new ControlP5(this);
  
  gui = new SpringGUI(this);
  gui.addList("deviceList", 60, 10, 400, 100);
  gui.addTextField("device", "", 100, offsetY-10, 380, 20);
  gui.addTextField("net","", 100, offsetY+20, 280, 20);
  gui.addTextField("pingres","", 100, offsetY+60, 100, 20);
  
   SampleListener lstn=new SampleListener(oscP5);
   try {
         jmdns = JmDNS.create();      
         jmdns.addServiceListener("_osc._udp.local.",lstn);
          
    } catch (IOException e) {
       e.printStackTrace();
    }
    
controlP5.addButton("ping",0,60,offsetY+55,30,30);
controlP5.addButton("Play_Pause",0,60,offsetY+100,90,40);
controlP5.addButton("start",0,170,offsetY+100,90,40);
controlP5.addButton("stop",0,280,offsetY+100,90,40);
controlP5.addButton("Mark",0,60,offsetY+160,90,40);
controlP5.addButton("MarkPos",0,170,offsetY+160,90,40);
controlP5.addButton("MarkJump",0,60,offsetY+240,90,40);
controlP5.addButton("No1Jump",0,60,offsetY+300,90,40);
controlP5.addButton("No2Jump",0,170,offsetY+300,90,40);
controlP5.addButton("No3Jump",0,280,offsetY+300,90,40);
controlP5.addButton("SecBack",0,60,offsetY+370,90,40);
controlP5.addButton("PosJump",0,60,offsetY+440,90,40);

markpos=controlP5.addTextfield("markPos(sec)",270,offsetY+160,100,30);
markpos.setText("60.0");

time= controlP5.addTextfield("sec(sec)",160,offsetY+370,100,30);
time.setText("15.0");

pos= controlP5.addTextfield("position(sec)",160,offsetY+440,100,30);
pos.setText("120.0");

markSlider=controlP5.addSlider("mark",0.0,1.0,0.5,400,offsetY+160,200,20);
posSlider=controlP5.addSlider("position",0.0,1.0,0.0,400,offsetY+440,200,20);
volSlider=controlP5.addSlider("volume",0.0,1.0,0.5,60,offsetY+500,100,20);

 // controlP5.addButton("clear",0,60,100,90,20);
 // controlP5.addButton("performTextfieldActions",0,60,50,150,20);
 // controlP5.addToggle("toggleAutoClear",true,220,50,100,20).setCaptionLabel("Auto Clear");
 // controlP5.addToggle("toggleKeepFocus",true,330,50,100,20).setCaptionLabel("Keep Focus");
  
//  controlP5.addButton("IPset",0,280,10,60,20);
}

void draw() {
  background(0);
}

void oscEvent(OscMessage theOscMessage) {
  /* print the address pattern and the typetag of the received OscMessage */
  print("### received an osc message.");
  print(" addrpattern: "+theOscMessage.addrPattern());
  println(" typetag: "+theOscMessage.typetag());
  
  if( theOscMessage.addrPattern().equals("/cbe/ping") ){
    gui.setText("pingres", "success");
  }
  else{
    gui.setText("pingres", "fail");
  }
  
  
  
  
}

void ping(int theValue){
  println("ping");
  gui.setText("pingres", "no response");
  OscMessage myMessage = new OscMessage("/cbe/ping");
  myMessage.add(0);
  oscP5.send(myMessage, myRemoteLocation); 
}

void Play_Pause(int theValue){
  
  OscMessage myMessage = new OscMessage("/cbe/play");

  myMessage.add(0);
 
  oscP5.send(myMessage, myRemoteLocation); 
   
}
void start(int theValue){
  OscMessage myMessage = new OscMessage("/cbe/play");
  myMessage.add(1);
  oscP5.send(myMessage, myRemoteLocation); 
}
void stop(int theValue){
  OscMessage myMessage = new OscMessage("/cbe/play");
  myMessage.add(2);
  oscP5.send(myMessage, myRemoteLocation); 
}
void Mark(int theValue){
  OscMessage myMessage = new OscMessage("/cbe/mark");
  myMessage.add(1);
  oscP5.send(myMessage, myRemoteLocation); 
}
void MarkPos(int theValue){
  OscMessage myMessage = new OscMessage("/cbe/markpos");
  String st=markpos.getText();
  float value=float(st);
  myMessage.add(value);
  oscP5.send(myMessage, myRemoteLocation); 
}
void MarkJump(int theValue){
  OscMessage myMessage = new OscMessage("/cbe/jump");
  myMessage.add(0);
  oscP5.send(myMessage, myRemoteLocation); 
}
void No1Jump(int theValue){
  OscMessage myMessage = new OscMessage("/cbe/jump");
  myMessage.add(1);
  oscP5.send(myMessage, myRemoteLocation); 
}
void No2Jump(int theValue){
  OscMessage myMessage = new OscMessage("/cbe/jump");
  myMessage.add(2);
  oscP5.send(myMessage, myRemoteLocation); 
}
void No3Jump(int theValue){
  OscMessage myMessage = new OscMessage("/cbe/jump");
  myMessage.add(3);
  oscP5.send(myMessage, myRemoteLocation); 
}

void SecBack(int theValue){
  String st=time.getText();
  float value=float(st);
  OscMessage myMessage = new OscMessage("/cbe/timeback");
  myMessage.add(value);
  oscP5.send(myMessage, myRemoteLocation); 
}

void PosJump(int theValue){
  String st=pos.getText();
  float value=float(st);
  OscMessage myMessage = new OscMessage("/cbe/posjump");
  myMessage.add(value);
  oscP5.send(myMessage, myRemoteLocation); 
}


void mark(float value) {
  OscMessage myMessage = new OscMessage("/cbe/markslider");
  myMessage.add(value);
  oscP5.send(myMessage, myRemoteLocation);
}
void position(float value) {
 OscMessage myMessage = new OscMessage("/cbe/posslider");
  myMessage.add(value);
  oscP5.send(myMessage, myRemoteLocation);
}
void volume(float value) {
  OscMessage myMessage = new OscMessage("/cbe/volslider");
  myMessage.add(value);
  oscP5.send(myMessage, myRemoteLocation);
}


void controlEvent(ControlEvent theEvent) {
  if(theEvent.controller() instanceof Textfield) {
    println("controlEvent: accessing a string from controller '"+theEvent.controller().name()+"': "+theEvent.controller().stringValue());
    // Textfield.isAutoClear() must be true
    print("controlEvent: trying to setText, ");
    ((Textfield)theEvent.controller()).setText("controlEvent: changing text.");
    if(((Textfield)theEvent.controller()).isAutoClear()==false) {
      println(" success!");
    } else {
      println(" but Textfield.isAutoClear() is false, could not setText here.");
    }
  }
}

void performTextfieldActions() {
  println("\n");
  // Textfield.getText();
  println("the current text of myTextfield: "+myTextfield.getText());
  println("the current value of textValue: "+textValue);
  // Textfield.setText();
  myTextfield.setText("changed the text of a textfield");
  println("changing text of myTextfield to: "+myTextfield.getText());
  // Textfield.getTextList();
  println("the textlist of myTextfield has "+myTextfield.getTextList().length+" items.");
  for(int i=0;i<myTextfield.getTextList().length;i++) {
    println("\t"+myTextfield.getTextList()[i]);
  }
  println("\n");
}




public void texting(String theText) {
   println(theText);
 
   myRemoteLocation = new NetAddress(theText,10000);
 
   myTextfield.setText(theText);
  // receiving text from controller texting
  println("a textfield event for controller 'texting': "+theText);
}


void handleEvent(String[] parameters) {
  
  if (parameters[1].equals("deviceList") && parameters[2].equals("selected")){
      
      String name=gui.getSelectedItem("deviceList");
      
    ServiceInfo service = jmdns.getServiceInfo(serviceType, name);
    if (service == null)   println("service not found");
     else                  jmdns.requestServiceInfo(serviceType, name);

      
    
    
    
  }
  
}

